const express = require('express');
const multer = require('multer');
const sharp = require('sharp');
const fs = require('fs');
const path = require('path');

const app = express();
const upload = multer({ dest: 'uploads/' });
const axios = require('axios');

async function validateToken(token) {
  try {
    const textoReverso = token.split('').reverse().join('');
    const response = await axios.get('https://totalplayrest.azurewebsites.net/api/imagen/token', {
      headers: {
        Authorization: `${textoReverso.replace(new RegExp(' ','g'),'+')}`
      }
    });


    if (response.data.status === 200) {
      return true;
    } else {
      return false;
    }
  } catch (error) {
    return false;
  }
}
// Ruta para subir la imagen o video con identificador
app.post('/api/upload/:id/:sn/:tipo', upload.single('file'), async (req, res) => {
  try {
    const { id, sn, tipo } = req.params;
    const { file } = req;

    const targetDirectory = path.join('uploads', id);
    createDirectory(targetDirectory);

    const fileName = `${id}_${sn}_${tipo}`;
    const filePath = path.join(targetDirectory, fileName);
    fs.renameSync(file.path, filePath + '.jpg');

    // Generar versiones de baja, mediana y alta escala sin marca de agua
    await generateScaledImage(filePath + '.jpg', targetDirectory, fileName, 'low', 0.5, false);
    await generateScaledImage(filePath + '.jpg', targetDirectory, fileName, 'medium', 0.75, false);
    await generateScaledImage(filePath + '.jpg', targetDirectory, fileName, 'high', 1, false);

    // Generar versiones de baja, mediana y alta escala con marca de agua
    await generateScaledImage(filePath + '.jpg', targetDirectory, fileName, 'low_watermark', 0.5, true);
    await generateScaledImage(filePath + '.jpg', targetDirectory, fileName, 'medium_watermark', 0.75, true);
    await generateScaledImage(filePath + '.jpg', targetDirectory, fileName, 'high_watermark', 1, true);

    res.send('Imágenes generadas exitosamente.');
  } catch (error) {
    console.log(error)
    res.status(500).send('Error al generar las imágenes.');
  }
});

// Ruta para obtener la imagen en diferentes tamaños con identificador
app.get('/api/image/:id', async (req, res) => {
  try {
    const { id } = req.params;
    const { scale, watermark, sn, tipo, token } = req.query;

    const targetDirectory = path.join('uploads', id);

    let suffix = scale || 'medium';
    if (watermark && watermark === '1') {
      suffix += '_watermark';
    }

    const scaledFilePath = './' + path.join(targetDirectory, `${id}_${sn}_${tipo}_${suffix}.jpg`);
    
    if(token){
        let valid = await validateToken(token)
        console.log(valid)
        if(valid){
            if (fs.existsSync(scaledFilePath)) {
                const filePath = path.resolve(scaledFilePath);
                console.log(filePath)
        
                res.sendFile(filePath, { root: '/' }, (err) => {
                    if (err) {
                        console.error(err);
                        res.status(404).send('File not found');
                    }
                });
            }else{
                res.status(500).send('Error al obtener la imagen.');
            }
        }else{
            res.status(500).send('Error al obtener la imagen.');
        }
    }else{
        res.status(500).send('Error al obtener la imagen.');
    }

  } catch (error) {
    console.log(error)
    res.status(500).send('Error al obtener la imagen.');
  }
});

// async function generateScaledImage(filePath, targetDirectory, fileName, suffix, scale, addWatermark) {
//   const scaledFilePath = path.join(targetDirectory, `${fileName}_${suffix}.jpg`);

//   const image = sharp(filePath);
//   const metadata = await image.metadata();
//   const width = Math.floor(metadata.width * scale);
//   const height = Math.floor(metadata.height * scale);

//   // Redimensionar la imagen a la escala deseada
//   await image.resize(width, height).toFile(scaledFilePath);

//   if (addWatermark) {
//     // Agregar marca de agua
//     const watermarkPath = 'watermark.png'; // Ruta del archivo de marca de agua
//     await sharp(scaledFilePath)
//       .composite([{ input: watermarkPath }])
//       .toFile(path.join(targetDirectory, `${fileName}_${suffix}_watermarked.jpg`));
//   }
// }
const watermarkPath = 'watermark.png'; // Ruta del archivo de marca de agua
const logoRepeat = 20; // Número de veces que se repetirá el logo

async function generateScaledImage(filePath, targetDirectory, fileName, suffix, scale, addWatermark) {
  const scaledFilePath = path.join(targetDirectory, `${fileName}_${suffix}.jpg`);

  const image = sharp(filePath);
  const metadata = await image.metadata();
  const width = Math.floor(metadata.width * scale);
  const height = Math.floor(metadata.height * scale);

  // Redimensionar la imagen principal a la escala deseada
  const resizedImage = await image.resize(width, height);

  if (addWatermark) {
    // Cargar la imagen de marca de agua
    const watermarkImage = sharp(watermarkPath);
    const watermarkMetadata = await watermarkImage.metadata();
    const watermarkWidth = 100; // Ajustar el tamaño del logo según tus necesidades
    const watermarkHeight = 100;

    // Generar varias posiciones aleatorias para la marca de agua sin superposición
    const watermarkPositions = [];
    for (let i = 0; i < logoRepeat; i++) {
      let position;
      do {
        position = {
          top: Math.floor(Math.random() * (height - watermarkHeight)),
          left: Math.floor(Math.random() * (width - watermarkWidth))
        };
      } while (watermarkPositions.some(p => isOverlap(position, p, watermarkWidth, watermarkHeight)));

      watermarkPositions.push(position);
    }

    // Combinar la imagen redimensionada con las marcas de agua en las posiciones aleatorias
    const compositeImage = await resizedImage.composite(
      watermarkPositions.map(position => ({
        input: watermarkPath,
        top: position.top,
        left: position.left
      }))
    );

    await compositeImage.toFile(path.join(targetDirectory, `${fileName}_${suffix}.jpg`));
  } else {
    // Guardar la imagen redimensionada sin marca de agua
    await resizedImage.toFile(scaledFilePath);
  }
}

// Función auxiliar para verificar si dos posiciones se superponen
function isOverlap(position1, position2, width, height) {
  const left1 = position1.left;
  const right1 = position1.left + width;
  const top1 = position1.top;
  const bottom1 = position1.top + height;

  const left2 = position2.left;
  const right2 = position2.left + width;
  const top2 = position2.top;
  const bottom2 = position2.top + height;

  return !(left2 > right1 || right2 < left1 || top2 > bottom1 || bottom2 < top1);
}



  
  

function createDirectory(directory) {
  if (!fs.existsSync(directory)) {
    fs.mkdirSync(directory, { recursive: true });
  }
}

app.listen(3000, () => {
  console.log('Servidor iniciado en el puerto 3000');
});

